@@ -347,7 +347,43 @@ class TestPAL(unittest.TestCase) :
         self.assertAlmostEqual( ev[2], -1.435296627515719e-7, 18 )
 
     def test_evp(self):
-        pass
+        vbex = np.array([ 1.6957348127008098514e-07,
+                         -9.1093446116039685966e-08,
+                         -3.9528532243991863036e-08 ])
+        pbex = np.array([-0.49771075259730546136,
+                         -0.80273812396332311359,
+                         -0.34851593942866060383])
+        vhex = np.array([ 1.6964379181455713805e-07,
+                         -9.1147224045727438391e-08,
+                         -3.9553158272334222497e-08])
+        phex = np.array([-0.50169124421419830639,
+                         -0.80650980174901798492,
+                          -0.34997162028527262212])
+
+        vbex2 = np.array([-0.0109187426811683,
+                          -0.0124652546173285,
+                          -0.0054047731809662])
+        pbex2 = np.array([-0.7714104440491060,
+                          +0.5598412061824225,
+                          +0.2425996277722475])
+        vhex2 = np.array([-0.0109189182414732,
+                          -0.0124718726844084,
+                          -0.0054075694180650])
+        phex2 = np.array([-0.7757238809297653,
+                          +0.5598052241363390,
+                          +0.2426998466481708])
+
+        (dvb, dpb, dvh, dph) = pal.evp( 2010.0, 2012.0 )
+        np.testing.assert_allclose( dvb, vbex, atol=1e-12 )
+        np.testing.assert_allclose( dpb, pbex, atol=1e-12 )
+        np.testing.assert_allclose( dvh, vhex, atol=1e-12 )
+        np.testing.assert_allclose( dph, phex, atol=1e-12 )
+
+        (dph, dvh, dpb, dvb) = pal.epv( 53411.52501161 )
+        np.testing.assert_allclose( dvb, vbex2, atol=1e-12 )
+        np.testing.assert_allclose( dpb, pbex2, atol=1e-12 )
+        np.testing.assert_allclose( dvh, vhex2, atol=1e-12 )
+        np.testing.assert_allclose( dph, phex2, atol=1e-12 )
 
     def test_fk45z(self):
         (r2000, d2000) = pal.fk45z( 1.2, -0.3, 1960 )
@@ -491,7 +527,7 @@ class TestPAL(unittest.TestCase) :
         ] )
 
         rmatn = pal.nut( 46012.32 )
-        np.testing.assert_array_almost_equal( rmatn, expected, 3 )
+        np.testing.assert_array_almost_equal( rmatn, expected, decimal=3 )
 
         (dpsi, deps, eps0) = pal.nutc( 54388.0 )
         self.assertAlmostEqual( eps0, 0.4090749229387258204, 14 )
@@ -508,7 +544,7 @@ class TestPAL(unittest.TestCase) :
         self.assertAlmostEqual( mmt["lat"], 0.5530735081550342238, 10 )
         self.assertAlmostEqual( mmt["height"], 2608, 10 )
 
-        self.assertEqual( len(obsdata), 83 )
+        self.assertEqual( len(obsdata), 85 )
 
     def test_pa(self):
         self.assertAlmostEqual( pal.pa( -1.567, 1.5123, 0.987 ),
@@ -516,7 +552,180 @@ class TestPAL(unittest.TestCase) :
         self.assertAlmostEqual( pal.pa( 0, 0.789, 0.789 ), 0, 12 )
 
     def test_planet(self):
-        pass
+        # palEl2ue
+        u = pal.el2ue( 50000, 1, 49000, 0.1, 2, 0.2,
+                       3, 0.05, 3, 0.003312 )
+        expectedue1 = np.array([1.000878908362435284,  -0.3336263027874777288,  50000.,
+                2.840425801310305210,   0.1264380368035014224, -0.2287711835229143197,
+                -0.01301062595106185195, 0.5657102158104651697,  0.2189745287281794885,
+                2.852427310959998500,  -0.01552349065435120900,
+                50000., 0.0])
+        np.testing.assert_allclose( u, expectedue1, atol=1e-12 )
+
+        # palPertel
+        (epoch, orbinc, anode, perih, aorq, e, aorl) = pal.pertel(2, 43000., 43200., 43000.,
+                                                                  0.2, 3, 4, 5, 0.02, 6)
+        self.assertAlmostEqual( epoch, 43200, 10 )
+        self.assertAlmostEqual( orbinc, 0.1995661466545422381, 7 )
+        self.assertAlmostEqual( anode, 2.998052737821591215, 7 )
+        self.assertAlmostEqual( perih, 4.009516448441143636, 6 )
+        self.assertAlmostEqual( aorq, 5.014216294790922323, 7 )
+        self.assertAlmostEqual( e, 0.02281386258309823607, 7 )
+        self.assertAlmostEqual( aorl, 0.01735248648779583748, 6 )
+
+        # palPertue
+        unew = pal.pertue( 50100, u )
+        expectedue3 = np.array([
+            1.000000000000000,
+            -0.3329769417028020949,
+            50100.,
+            2.638884303608524597,
+            1.070994304747824305,
+            0.1544112080167568589,
+            -0.2188240619161439344,
+            0.5207557453451906385,
+            0.2217782439275216936,
+            2.852118859689216658,
+            0.01452010174371893229,
+            50100., 0. ])
+        np.testing.assert_allclose( unew, expectedue3, atol=1e-12 )
+
+        # palPlanel
+        pv = pal.planel( 50600, 2, 50500, 0.1, 3, 5, 2, 0.3, 4, 0 )
+        expectedpv2 = np.array ([1.947628959288897677,
+            -1.013736058752235271,
+            -0.3536409947732733647,
+            2.742247411571786194e-8,
+            1.170467244079075911e-7,
+            3.709878268217564005e-8])
+        np.testing.assert_allclose( pv, expectedpv2, atol=1e-12 )
+
+        # palPlanet
+        self.assertRaises( ValueError, pal.planet, 1e6, 0 )
+        self.assertRaises( ValueError, pal.planet, 1e6, 9 )
+
+        pv = pal.planet( -320000, 3 )
+        self.assertAlmostEqual( pv[0], 0.9308038666827242603, 11)
+        self.assertAlmostEqual( pv[1], 0.3258319040252137618, 11)
+        self.assertAlmostEqual( pv[2], 0.1422794544477122021, 11)
+        self.assertAlmostEqual( pv[3], -7.441503423889371696e-8, 17)
+        self.assertAlmostEqual( pv[4], 1.699734557528650689e-7, 17)
+        self.assertAlmostEqual( pv[5], 7.415505123001430864e-8, 17)
+
+        pv = pal.planet( 43999.9, 1 )
+        self.assertAlmostEqual( pv[0], 0.2945293959257422246, 11)
+        self.assertAlmostEqual( pv[1], -0.2452204176601052181, 11)
+        self.assertAlmostEqual( pv[2], -0.1615427700571978643, 11)
+        self.assertAlmostEqual( pv[3], 1.636421147459047057e-7, 18)
+        self.assertAlmostEqual( pv[4], 2.252949422574889753e-7, 18)
+        self.assertAlmostEqual( pv[5], 1.033542799062371839e-7, 18)
+
+        # palPlante
+        (ra, dec, r) = pal.plante( 50600., -1.23, 0.456, 2, 50500.,
+                                   0.1, 3., 5., 2., 0.3, 4., 0.0 )
+        self.assertAlmostEqual( ra, 6.222958101333794007, 6 )
+        self.assertAlmostEqual( dec, 0.01142220305739771601, 6 )
+        self.assertAlmostEqual( r, 2.288902494080167624, 8 )
+
+        # palPlantu
+        u = np.array( [1.0005, -0.3, 55000., 2.8, 0.1, -0.2,
+                       -0.01, 0.5, 0.22, 2.8, -0.015, 55001., 0.0] )
+        (ra, dec, r) = pal.plantu( 55001., -1.23, 0.456, u )
+        self.assertAlmostEqual( ra, 0.3531814831241686647, 6 )
+        self.assertAlmostEqual( dec, 0.06940344580567131328, 6 )
+        self.assertAlmostEqual( r, 3.031687170873274464, 8 )
+
+        # palPv2el
+        pv = np.array( [ 0.3, -0.2, 0.1, -0.9e-7, 0.8e-7, -0.7e-7 ] )
+        (jform, epoch, orbinc, anode, perih, aorq, e, aorl, dm ) = pal.pv2el( pv, 50000, 0.00006, 1 )
+        self.assertEqual( jform, 1 )
+        self.assertAlmostEqual( epoch, 50000, 10 )
+        self.assertAlmostEqual( orbinc, 1.52099895268912, 12 )
+        self.assertAlmostEqual( anode, 2.720503180538650, 12 )
+        self.assertAlmostEqual( perih, 2.194081512031836, 12 )
+        self.assertAlmostEqual( aorq, 0.2059371035373771, 12 )
+        self.assertAlmostEqual( e, 0.9866822985810528, 12 )
+        self.assertAlmostEqual( aorl, 0.2012758344836794, 12 )
+        self.assertAlmostEqual( dm, 0.184074050795182, 12 )
+
+        # palPv2ue
+        expectedue2 = np.array( [1.00006, -4.856142884511782, 50000., 0.3, -0.2,
+                                0.1,  -0.4520378601821727,  0.4018114312730424,
+                                -.3515850023639121, 0.3741657386773941,
+                                -0.2511321445456515, 50000., 0.] )
+        u = pal.pv2ue( pv, 50000., 0.00006 )
+        np.testing.assert_allclose( u, expectedue2, atol=1e-12 )
+
+        # Planets
+        (ra, dec, diam) = pal.rdplan( 40999.9, 0, 0.1, -0.9 )
+        self.assertAlmostEqual( ra, 5.772270359389275837, 6 )
+        self.assertAlmostEqual( dec, -0.2089207338795416192, 7 )
+        self.assertAlmostEqual( diam, 9.415338935229717875e-3, 10 )
+
+        (ra, dec, diam) = pal.rdplan( 41999.9, 1, 1.1, -0.9 )
+        self.assertAlmostEqual( ra, 3.866363420052936653, 6 )
+        self.assertAlmostEqual( dec, -0.2594430577550113130, 7 )
+        self.assertAlmostEqual( diam, 4.638468996795023071e-5, 14 )
+
+        (ra, dec, diam) = pal.rdplan( 42999.9, 2, 2.1, 0.9 )
+        self.assertAlmostEqual( ra, 2.695383203184077378, 6 )
+        self.assertAlmostEqual( dec, 0.2124044506294805126, 7 )
+        self.assertAlmostEqual( diam, 4.892222838681000389e-5, 14 )
+
+        (ra, dec, diam) = pal.rdplan( 43999.9, 3, 3.1, 0.9 )
+        self.assertAlmostEqual( ra, 2.908326678461540165, 6 )
+        self.assertAlmostEqual( dec, 0.08729783126905579385, 7 )
+        self.assertAlmostEqual( diam, 8.581305866034962476e-3, 7 )
+
+        (ra, dec, diam) = pal.rdplan( 44999.9, 4, -0.1, 1.1 )
+        self.assertAlmostEqual( ra, 3.429840787472851721, 6 )
+        self.assertAlmostEqual( dec, -0.06979851055261161013, 7 )
+        self.assertAlmostEqual( diam, 4.540536678439300199e-5, 14 )
+
+        (ra, dec, diam) = pal.rdplan( 45999.9, 5, -1.1, 0.1 )
+        self.assertAlmostEqual( ra, 4.864669466449422548, 6 )
+        self.assertAlmostEqual( dec, -0.4077714497908953354, 7 )
+        self.assertAlmostEqual( diam, 1.727945579027815576e-4, 14 )
+
+        (ra, dec, diam) = pal.rdplan( 46999.9, 6, -2.1, -0.1 )
+        self.assertAlmostEqual( ra, 4.432929829176388766, 6 )
+        self.assertAlmostEqual( dec, -0.3682820877854730530, 7 )
+        self.assertAlmostEqual( diam, 8.670829016099083311e-5, 14 )
+
+        (ra, dec, diam) = pal.rdplan( 47999.9, 7, -3.1, -1.1 )
+        self.assertAlmostEqual( ra, 4.894972492286818487, 6 )
+        self.assertAlmostEqual( dec, -0.4084068901053653125, 7 )
+        self.assertAlmostEqual( diam, 1.793916783975974163e-5, 14 )
+
+        (ra, dec, diam) = pal.rdplan( 48999.9, 8, 0, 0 )
+        self.assertAlmostEqual( ra, 5.066050284760144000, 6 )
+        self.assertAlmostEqual( dec, -0.3744690779683850609, 7 )
+        self.assertAlmostEqual( diam, 1.062210086082700563e-5, 14 )
+
+        # palUe2el
+        (jform, epoch, orbinc, anode, perih, aorq, e, aorl, dm ) = pal.ue2el( u, 1 )
+        self.assertEqual( jform, 1 )
+        self.assertAlmostEqual( epoch, 50000.00, 10 )
+        self.assertAlmostEqual( orbinc, 1.520998952689120, 12 )
+        self.assertAlmostEqual( anode, 2.720503180538650, 12 )
+        self.assertAlmostEqual( perih, 2.194081512031836, 12 )
+        self.assertAlmostEqual( aorq, 0.2059371035373771, 12 )
+        self.assertAlmostEqual( e, 0.9866822985810528, 12 )
+        self.assertAlmostEqual( aorl, 0.2012758344836794, 12 )
+
+        # palUe2pv
+        (u2, pv) = pal.ue2pv( 50010., u )
+
+        # Update final two elements of the test UE array
+        expectedue2[11] = 50010.;
+        expectedue2[12] = 0.7194308220038886856;
+        np.testing.assert_allclose( u2, expectedue2, atol=1e-12 )
+
+        expectedpv = np.array( [
+            0.07944764084631667011, -0.04118141077419014775,
+            0.002915180702063625400, -0.6890132370721108608e-6,
+            0.4326690733487621457e-6, -0.1763249096254134306e-6 ] )
+        np.testing.assert_allclose( pv, expectedpv, atol=1e-12 )
 
     def test_pm(self):
         (ra, dec) = pal.pm( 5.43, -0.87, -0.33e-5, 0.77e-5, 0.7,
@@ -572,6 +781,60 @@ class TestPAL(unittest.TestCase) :
     def test_ranorm(self):
         self.assertAlmostEqual( pal.dranrm( -0.1 ), 6.183185307179587, 12 )
 
+    def test_ref(self):
+        self.assertAlmostEqual( pal.refro(1.4, 3456.7, 280, 678.9, 0.9, 0.55,
+                                -0.3, 0.006, 1e-9 ),
+                                0.00106715763018568, 12 )
+        self.assertAlmostEqual( pal.refro(1.4, 3456.7, 280, 678.9, 0.9, 1000,
+                                -0.3, 0.006, 1e-9),
+                                0.001296416185295403, 12 )
+
+        (refa, refb) = pal.refcoq( 275.9, 709.3, 0.9, 101 )
+        self.assertAlmostEqual( refa, 2.324736903790639e-4, 12 )
+        self.assertAlmostEqual( refb, -2.442884551059e-7, 15 )
+
+        (refa, refb) = pal.refco( 2111.1, 275.9, 709.3, 0.9, 101,
+                                -1.03, 0.0067, 1e-12 )
+        self.assertAlmostEqual( refa, 2.324673985217244e-4, 12 )
+        self.assertAlmostEqual( refb, -2.265040682496e-7, 15 )
+
+        (refa, refb) = pal.refcoq( 275.9, 709.3, 0.9, 0.77 )
+        self.assertAlmostEqual( refa, 2.007406521596588e-4, 12 )
+        self.assertAlmostEqual( refb, -2.264210092590e-7, 15 )
+
+        (refa, refb) = pal.refco( 2111.1, 275.9, 709.3, 0.9, 0.77,
+                                -1.03, 0.0067, 1e-12 )
+        self.assertAlmostEqual( refa, 2.007202720084551e-4, 12 )
+        self.assertAlmostEqual( refb, -2.223037748876e-7, 15 )
+
+        (refa2, refb2) = pal.atmdsp( 275.9, 709.3, 0.9, 0.77,
+                                     refa, refb, 0.5 )
+        self.assertAlmostEqual( refa2, 2.034523658888048e-4, 12 )
+        self.assertAlmostEqual( refb2, -2.250855362179e-7, 15 )
+
+        vu = pal.dcs2c( 0.345, 0.456 )
+        vr = pal.refv( vu, refa, refb )
+        self.assertAlmostEqual( vr[0], 0.8447487047790478, 12 )
+        self.assertAlmostEqual( vr[1], 0.3035794890562339, 12 )
+        self.assertAlmostEqual( vr[2], 0.4407256738589851, 12 )
+
+        vu = pal.dcs2c( 3.7, 0.03 )
+        vr = pal.refv( vu, refa, refb )
+        self.assertAlmostEqual( vr[0], -0.8476187691681673, 12 )
+        self.assertAlmostEqual( vr[1], -0.5295354802804889, 12 )
+        self.assertAlmostEqual( vr[2], 0.0322914582168426, 12 )
+
+        zr = pal.refz( 0.567, refa, refb )
+        self.assertAlmostEqual( zr, 0.566872285910534, 12 )
+
+        zr = pal.refz( 1.55, refa, refb )
+        self.assertAlmostEqual( zr, 1.545697350690958, 12 )
+
+    def test_refc(self): # This is the SOFA test
+        (refa, refb) = pal.refcoq( 10.0+273.15, 800.0, 0.9, 0.4)
+        self.assertAlmostEqual( refa, 0.2264949956241415009e-3, 15 )
+        self.assertAlmostEqual( refb, -0.2598658261729343970e-6, 18 )
+
     def test_rv(self):
         self.assertAlmostEqual( pal.rverot( -0.777, 5.67, -0.3, 3.19 ),
                                  -0.1948098355075913, 6 )
